clear all

phiz=1;      %Phi a function of z? 1 if yes, 0 if no.

% Parameters and functions
rho = 1-1/6;
alpha=.64;
sigma = 3.04;
nu =.56;
N = 100;
psi=.46;


means=log(1.7);
vars=0.4;
ns = 500;
smin=0.01;
smax=exp(means+4*vars);
jump=(smax-smin)/(ns-1);
sgrid=smin:jump:smax;
mu=normpdf(log(sgrid),means,vars)/sum(normpdf(log(sgrid),means,vars));
mucdf=normcdf(log(sgrid),means,vars);
[med,inmed]=min(abs((normcdf(log(sgrid),means,vars)'-.5)));
[t10,int10]=min(abs((normcdf(log(sgrid),means,vars)'-.9)));
[t1,int1]=min(abs((normcdf(log(sgrid),means,vars)'-.99)));

tfpdrop=.90;
nz=2;
zh=1;
zgrid=[tfpdrop*zh zh];

% Initial Guess
mguess=1;
stilde0=(1/N)*sum(mguess*mu.*(sgrid.^(1/(1-alpha))));
w0=.1;
Phi0=sum(mu.*phifunpcomp2(mguess*ones(1,ns),nu,psi*w0,zgrid(1),phiz));

guess=[w0
       stilde0
       Phi0];

options=optimset('display','iter','maxfunevals',100000,'maxiter',1000,'largescale','off','tolX',1e-50,'TolCon',1e-50,'tolfun',1e-50);
zi=1;
while zi<=nz;
    sol(:,zi)=fsolve(@sysperfcomp,guess,options,zgrid(zi),alpha,sigma,N,rho,nu,sgrid,psi,mu,ns,phiz);
    zi=zi+1;%guess=sol(:,zi-1);
end

sol=real(sol);
for zi=1:nz;
    w(zi)=sol(1,zi);
    stilde(zi)=sol(2,zi);
    Phi(zi)=sol(3,zi);
    D(zi)=w(zi)*(((1-Phi(zi))/alpha)-1);
    P(zi)=(((exp((sigma^2)/2))*N)^((alpha*rho-1)/rho))*w(zi)/(alpha*zgrid(zi))*((1-Phi(zi))/stilde(zi))^(1-alpha);
    a(zi)=afunpcomp(P(zi),w(zi),D(zi),stilde(zi),zgrid(zi),alpha,rho);
    b(zi)=bfunpcomp(a(zi),sigma,zgrid(zi),w(zi),alpha);
    m(:,zi)=mfunpcomp(sgrid,b(zi),nu,psi,w(zi),zgrid(zi),N,alpha,ns,phiz);
    l(:,zi)=m(:,zi)'.*exp(sigma^2/2).*a(zi).^(1/(1-alpha)).*sgrid.^(1/(1-alpha)).*(zgrid(zi)*alpha/w(zi))^(1/(1-alpha));
    phi(:,zi)=phifunpcomp2(m(:,zi),nu,w(zi)*psi,zgrid(zi),phiz);   %expenditure in phi
    rev(:,zi)=exp(sigma^2/2).*m(:,zi).*(zgrid(zi).*sgrid'.*a(zi).*((alpha/w(zi))^alpha)).^(1/(1-alpha));
    vol(:,zi)=((exp((1-alpha)^2*sigma^2)-1)./m(:,zi)).^(1/2);
end
l=l+[phi(:,1)./w(1) phi(:,2)./w(2)];

% Statistics
L=l.*[mu' mu'];
Y=sum(rev.*[mu' mu'])./P;                 % Total output
TFP=Y(1)/Y(2);                            % Measured TFP drop
multtfp=(1-TFP)/(1-zgrid(1)/zgrid(2));    % Endogenous multiplication from tfp (z) to measured TFP
exp_rev=sum(phi./rev.*[mu' mu']);         % Average expenditures to revenue ratio
avgvoldif=sum(mu'.*(vol(:,2)-vol(:,1)))/sum(mu'.*vol(:,1));  % Average volatility difference as a % of volatility in bad times
medvol=vol(inmed,:);                      % Median volatility

avgdata=20.39;
distdata=[.61 .176 .1068 .08875 .0152 .00311]';
%avgdata=1;
% Distribution of employment over firm's size
sizel=[5/avgdata 10/avgdata 20/avgdata 100/avgdata 500/avgdata]';
for in=1:1:max(size(sizel))
    distl0(:,in)=sum([mu' mu'].*(l<=sizel(in)));
end
distl0(:,max(size(sizel))+1)=1;
distl(:,1)=distl0(:,1);
for in=2:1:max(size(distl0))
distl(:,in)=distl0(:,in)-distl0(:,in-1);
end
[distdata distl(2,:)'];

% Figures

figure;
subplot(2,1,1);
fig=plot(sgrid,m(:,1),'-',sgrid,m(:,2),':');
xlabel('productivity (s)')
ylabel('# markets served')
legend('low','high')
set(fig,'lineWidth',2)
subplot(2,1,2);
fig2=plot(sgrid,mu);
xlabel('productivity (s)')
ylabel('% of firms')
set(fig,'lineWidth',2)

figure;
plot(sgrid,((m(:,2)./m(:,1)).^.5)-1)
xlabel('productivity (s)')
ylabel('% Increase in CV')

%simulation

T=100;
nf=100000;   %number of firms drawn
nfs=ceil(mu'*nf);  %number of firms by productivity 
tfp0=rand(1,T);
tfp1=(tfp0>=0.8)*1+(tfp0<=0.8)*2;

%matrix with firm level productivity s
%smat=zeros(nf,T);
inf=1;
while inf<=nf;
    ins=1;
    while ins<=ns;
        insn=1;
        while insn<=nfs(ins)
            smat(inf,:)=sgrid(ins)*ones(1,T);
            sinmat(inf,:)=ins*ones(1,T);
            inf=inf+1;
            insn=insn+1;
        end
        ins=ins+1;
    end
end

%# markets each firm supplies 
dim=size(smat);
dimf=dim(1);
mmat=zeros(dimf,T);
inf=1;
while inf<=dimf;
    int=1;
    while int<=T
    nmat(inf,int)=m((sinmat(inf,int)),tfp1(int));
    int=int+1;
    end
    inf=inf+1;
end

% matrices with sales, employees, and expenditures
inf=1;
while inf<=dimf
tfpm(inf,:)=tfp1(:);
inf=inf+1;
end

intexp=((nmat-1).^(1+nu).*psi).*w(tfpm)./zgrid(tfpm);

int=1;
while int<=T;
    inf=1;
    while inf<=dimf;
        emp(inf,int)=sum(exp(sigma^2*randn(nmat(inf,int),1)))*(w(tfpm(inf,int))/(zgrid(tfpm(inf,int))*alpha*a(tfpm(inf,int))*smat(inf,int)))^(1/(alpha-1))+intexp(inf,int)/w(tfpm(inf,int)) ;
        sales(inf,int)=sum(exp(sigma^2*randn(nmat(inf,int),1)))*((w(tfpm(inf,int))/alpha)^(alpha/(alpha-1)))*(a(tfpm(inf,int))*zgrid(tfpm(inf,int))*smat(inf,int))^(1/(1-alpha));
        inf=inf+1;
    end
    int=int+1;
end

dsales=log(sales(:,2:T))-log(sales(:,1:T-1));
emp2=emp(:,1:T-1);
intexp2=intexp(:,2:T);
smat2=smat(:,1:T-1);
gdp=zgrid(tfpm(:,1:T-1));
nmat2=nmat(:,2:T);

dsales3=dsales(:,1);
emp3=emp2(:,1);
intexp3=intexp2(:,1);
smat3=smat2(:,1);
gdp3=gdp(:,1);
nmat3=nmat2(:,1);

in=2;
while in<=T-1
    dsales3=[dsales3
            dsales(:,in)];
        emp3=[emp3
            emp2(:,in)];
        intexp3=[intexp3
            intexp2(:,in)];
        smat3=[smat3
            smat2(:,in)];
        gdp3=[gdp3
            gdp(:,in)];
        nmat3=[nmat3
            nmat2(:,in)];
        in=in+1;
end
    
[b1,bint1,r1,rint1]=regress(dsales3,[ones(size(smat3)) gdp3 log(emp3) smat3]);

lerr=log(r1.^2);


perc=.95;

dsalespred=b1(1)*gdp+b1(2)*log(emp2)+b1(3)*smat2;
errmat=dsales-dsalespred;
errmat0=errmat;
errmat=log(errmat.^2);
errmat2=log(errmat);

dtfp=zgrid(tfp1(2:T))-zgrid(tfp1(1:T-1));

N=floor(dimf*1)-floor(dimf*perc);

terrmat0=errmat(max(1,floor(dimf*perc)):floor(dimf*1),:);
tsmat0=smat2(max(1,floor(dimf*perc)):floor(dimf*1),:);
temp0=emp2(max(1,floor(dimf*perc)):floor(dimf*1),:);
tgdp0=gdp(max(1,floor(dimf*perc)):floor(dimf*1),:);
tnmat0=nmat2(max(1,floor(dimf*perc)):floor(dimf*1),:);
tintexp0=intexp2(max(1,floor(dimf*perc)):floor(dimf*1),:);


terrmat1=terrmat0(:,1);
tsmat1=tsmat0(:,1);
tgdp1=tgdp0(:,1);
tnmat1=tnmat0(:,1);
tintexp1=tintexp0(:,1);
temp1=temp0(:,1);


in=2;
while in<=T-1
    terrmat1=[terrmat1
            terrmat0(:,in)];
    tsmat1=[tsmat1
            tsmat0(:,in)];
    tgdp1=[tgdp1
            tgdp0(:,in)];
    tnmat1=[tnmat1
            tnmat0(:,in)];
    tintexp1=[tintexp1
            tintexp0(:,in)];
        temp1=[temp1
            temp0(:,in)];
        in=in+1;
end


% Correlations in the paper Table 3
[ccA ccAp]=corrcoef(std(errmat0(max(1,floor(dimf*perc)):floor(dimf*1),:)),zgrid(tfp1(1,2:T)));
[ccB ccBp]=corrcoef(median(errmat(:,:)),zgrid(tfp1(1,2:T)));
[cc1 cc1p]=corrcoef(std(errmat0(max(1,floor(dimf*perc)):floor(dimf*1),:)),zgrid(tfp1(1,2:T)));
[cc3 cc3p]=corrcoef(median(errmat(max(1,floor(dimf*perc)):floor(dimf*1),:)),zgrid(tfp1(1,2:T)));

% Regressions in the paper table 7
[b1,bint1]=regress(lerr,[smat3 gdp3 log(nmat3)]);
mdl1=fitlm([smat3 gdp3 log(nmat3)],lerr,'intercept', false);

[b2,bint2]=regress(terrmat1,[tsmat1 tgdp1 log(tintexp1+.0000001)]);
mdl2=fitlm([tsmat1 tgdp1 log(tintexp1+.0000001)],terrmat1,'intercept', false);

[b3,bint3]=regress(terrmat1,[tsmat1 tgdp1 log(tnmat1)]);
mdl3=fitlm([tsmat1 tgdp1 log(tnmat1)],terrmat1,'intercept', false);

[b4,bint4]=regress(lerr,[smat3 gdp3  log(intexp3+.0000001)]);
mdl4=fitlm([smat3 gdp3 log(intexp3+.0000001)],lerr,'intercept', false);


% Model numbers in section 6.3.
[text63a text63asig]=corrcoef(median(exp(errmat(99005:100258,:))),zgrid(tfp1(1,2:T)));
[text63b text63bsig]=corrcoef(median(errmat(95245:96498,:)),zgrid(tfp1(1,2:T)));

% Footnote 43
[ft43a ft43asig]=corrcoef(median(exp(errmat(1:25065,:))),zgrid(tfp1(1,2:T)));
[ft43b ft43bsig]=corrcoef(median(exp(errmat(75193:100258,:))),zgrid(tfp1(1,2:T)));